﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using NiVE2.Plugin.Interface;
using NiVE2.Plugin.Property;
using NiVE2.Plugin.Controls;
using NiVE2.Plugin.Utils;
using NiVE2.Drawing;
using NiVE2.Drawing.Drawing2D;
using NiVE2.Utils;
using NiVE2.Utils.Cache;
using NiVE2.Utils.Threading;

namespace ruche.nive2.effects
{
    /// <summary>
    /// ブレンド処理を行うNiVE2エフェクトクラス。
    /// </summary>
    [UseLayer]
    public class Blend : EffectBase
    {
        /// <summary>
        /// ブレンド種別名配列。
        /// </summary>
        private static readonly string[] BlendTypeNames =
            new string[]
                {
                    "通常",
                    "置換",
                    "加算",
                    "減算",
                    "乗算",
                    "スクリーン",
                    "オーバーレイ",
                    "ハードライト",
                    "ソフトライト",
                    "ビビッドライト",
                    "リニアライト",
                    "ピンライト",
                    "覆い焼き(カラー)",
                    "覆い焼き(リニア)",
                    "焼き込み(カラー)",
                    "焼き込み(リニア)",
                    "比較(暗)",
                    "比較(明)",
                    "差の絶対値",
                    "除外",
                    "色相",
                    "彩度",
                    "カラー",
                    "明度",
                };

        /// <summary>
        /// 反転の種別名配列。
        /// </summary>
        private static readonly string[] FlipTypeNames =
            new string[]
                {
                    "なし",
                    "左右反転",
                    "上下反転",
                    "左右反転＆上下反転",
                };

        /// <summary>
        /// 反転の種別を表す列挙。
        /// </summary>
        private enum FlipType
        {
            None = 0,
            Horizontal,
            Vertical,
            Both,
        };

        /// <summary>
        /// 回転の基準点種別名配列。
        /// </summary>
        private static readonly string[] RotateBaseTypeNames =
            new string[]
                {
                    "直接指定",
                    "元レイヤーの左上端",
                    "元レイヤーの中央上端",
                    "元レイヤーの右上端",
                    "元レイヤーの左端中央",
                    "元レイヤーの中心",
                    "元レイヤーの右端中央",
                    "元レイヤーの左下端",
                    "元レイヤーの中央下端",
                    "元レイヤーの右下端",
                };

        /// <summary>
        /// 回転の基準点種別を表す列挙。
        /// </summary>
        private enum RotateBaseType
        {
            Absolute = 0,
            TopLeft,
            TopCenter,
            TopRight,
            CenterLeft,
            Center,
            CenterRight,
            BottomLeft,
            BottomCenter,
            BottomRight,
        };

        /// <summary>
        /// グラデーション方向名配列。
        /// </summary>
        private static readonly string[] GradientDirNames =
            new string[]
                {
                    "上から下",
                    "左から右",
                    "左上から右下",
                    "右上から左下",
                };

        /// <summary>
        /// グラデーション方向を表す列挙。
        /// </summary>
        private enum GradientDirection
        {
            TopToBottom = 0,
            LeftToRight,
            TopLeftToBottomRight,
            TopRightToBottomLeft,
        };

        /// <summary>
        /// 時間指定種別名配列。
        /// </summary>
        private static readonly string[] RenderTimeTypeNames =
            new string[]
                {
                    "指定しない",
                    "固定",
                    "オフセット",
                };

        /// <summary>
        /// 時間指定種別を表す列挙。
        /// </summary>
        private enum RenderTimeType
        {
            None = 0,
            Fixed,
            Offset,
        };

        /// <summary>
        /// 対象レイヤ。
        /// </summary>
        private ILayer _layer = null;

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public Blend()
        {
        }

        /// <summary>
        /// 指定したアイテムコードのレイヤーを検索して返す。
        /// </summary>
        /// <param name="itemCode">アイテムコード。</param>
        /// <returns>レイヤー。見つからなかった場合は null 。</returns>
        private ILayer FindLayer(int itemCode)
        {
            foreach (ILayer l in _layer.Composition.GetLayer())
            {
                if (l.ItemCode == itemCode)
                {
                    return l;
                }
            }
            return null;
        }

        /// <summary>
        /// 画像の不透明度情報を保持したまま単色で塗り潰す。
        /// </summary>
        /// <param name="target">画像。</param>
        /// <param name="color">色。</param>
        /// <param name="alpha">色の不透明度。</param>
        private void FillBitmapBySolidColor(NBitmap target, Color color, double alpha)
        {
            byte[] data = target.GetData();

            alpha = Math.Min(Math.Max(alpha, 0.0), 1.0);
            if (alpha == 1.0)
            {
                // 完全不透明
                Parallel.ForStep4(0, data.Length, (i) =>
                {
                    data[i] = color.B;
                    data[i + 1] = color.G;
                    data[i + 2] = color.R;
                });
            }
            else if (alpha > 0.0)
            {
                // 半透明
                double invAlpha = 1.0 - alpha;
                Parallel.ForStep4(0, data.Length, (i) =>
                {
                    data[i] = (byte)(color.B * alpha + data[i] * invAlpha + 0.5);
                    data[i + 1] = (byte)(color.G * alpha + data[i + 1] * invAlpha + 0.5);
                    data[i + 2] = (byte)(color.R * alpha + data[i + 2] * invAlpha + 0.5);
                });
            }
        }

        /// <summary>
        /// 画像の不透明度情報を保持したままグラデーション色で塗り潰す。
        /// </summary>
        /// <param name="target">画像。</param>
        /// <param name="baseColor">ベース色。</param>
        /// <param name="baseAlpha">ベース不透明度。</param>
        /// <param name="gradientColor">グラデーション色。</param>
        /// <param name="gradientAlpha">グラデーション不透明度。</param>
        /// <param name="gradientDir">グラデーション方向。</param>
        private void FillBitmapByGradientColor(
            NBitmap target,
            Color baseColor,
            double baseAlpha,
            Color gradientColor,
            double gradientAlpha,
            GradientDirection gradientDir)
        {
            // グラデーションの開始位置と終端位置を設定
            Point begPos = Point.Empty, endPos = Point.Empty;
            switch (gradientDir)
            {
                case GradientDirection.TopToBottom:
                    endPos.Y = target.Height - 1;
                    break;

                case GradientDirection.LeftToRight:
                    endPos.X = target.Width - 1;
                    break;

                case GradientDirection.TopLeftToBottomRight:
                    endPos.X = target.Width - 1;
                    endPos.Y = target.Height - 1;
                    break;

                case GradientDirection.TopRightToBottomLeft:
                    begPos.X = target.Width - 1;
                    endPos.Y = target.Height - 1;
                    break;
            }

            // 塗り潰し
            byte[] data = target.GetData();
            int lenX = target.Width * 4;
            baseAlpha = Math.Min(Math.Max(baseAlpha, 0.0), 1.0);
            gradientAlpha = Math.Min(Math.Max(gradientAlpha, 0.0), 1.0);
            if (baseAlpha == 1.0 && gradientAlpha == 1.0)
            {
                // 完全不透明
                Parallel.For(0, target.Height, (y) =>
                {
                    double gradRace, revRate;

                    Point p = new Point(0, y);
                    int bi = lenX * y;
                    for (p.X = 0; p.X < target.Width; ++p.X, bi += 4)
                    {
                        // グラデーション割合算出
                        gradRace = CalcGradientRate(p, begPos, endPos, gradientDir);
                        revRate = 1.0 - gradRace;

                        // 距離割合から色決定
                        data[bi] = (byte)(baseColor.B * revRate + gradientColor.B * gradRace + 0.5);
                        data[bi + 1] = (byte)(baseColor.G * revRate + gradientColor.G * gradRace + 0.5);
                        data[bi + 2] = (byte)(baseColor.R * revRate + gradientColor.R * gradRace + 0.5);
                    }
                });
            }
            else if (baseAlpha > 0.0 || gradientAlpha > 0.0)
            {
                // 半透明
                Parallel.For(0, target.Height, (y) =>
                {
                    byte b, g, r;
                    double alpha, invAlpha, gradRace, revRate;

                    Point p = new Point(0, y);
                    int bi = lenX * y;
                    for (p.X = 0; p.X < target.Width; ++p.X, bi += 4)
                    {
                        // グラデーション割合算出
                        gradRace = CalcGradientRate(p, begPos, endPos, gradientDir);
                        revRate = 1.0 - gradRace;

                        // 距離割合から色と不透明度決定
                        b = (byte)(baseColor.B * revRate + gradientColor.B * gradRace + 0.5);
                        g = (byte)(baseColor.G * revRate + gradientColor.G * gradRace + 0.5);
                        r = (byte)(baseColor.R * revRate + gradientColor.R * gradRace + 0.5);
                        alpha = baseAlpha * revRate + gradientAlpha * gradRace;
                        invAlpha = 1.0 - alpha;

                        // 色算出
                        data[bi] = (byte)(b * alpha + data[bi] * invAlpha + 0.5);
                        data[bi + 1] = (byte)(g * alpha + data[bi + 1] * invAlpha + 0.5);
                        data[bi + 2] = (byte)(r * alpha + data[bi + 2] * invAlpha + 0.5);
                    }
                });
            }
        }

        /// <summary>
        /// 現在位置のグラデーション割合を算出する。
        /// </summary>
        /// <param name="pos">現在位置。</param>
        /// <param name="beginPos">グラデーション開始位置。</param>
        /// <param name="endPos">グラデーション終了位置。</param>
        /// <param name="gradientDir">グラデーション方向。</param>
        /// <returns>
        /// グラデーション割合。 0.0 ～ 1.0 。終了位置に近いほど値が大きくなる。
        /// </returns>
        double CalcGradientRate(
            Point pos, Point beginPos, Point endPos, GradientDirection gradientDir)
        {
            // 開始点および終了点からの距離算出
            double beginDist, endDist;
            switch (gradientDir)
            {
                case GradientDirection.TopToBottom:
                    beginDist = Math.Abs(beginPos.Y - pos.Y);
                    endDist = Math.Abs(endPos.Y - pos.Y);
                    break;

                case GradientDirection.LeftToRight:
                    beginDist = Math.Abs(beginPos.X - pos.X);
                    endDist = Math.Abs(endPos.X - pos.X);
                    break;

                case GradientDirection.TopLeftToBottomRight:
                case GradientDirection.TopRightToBottomLeft:
                default:
                    beginDist = CalcDistance(beginPos, pos);
                    endDist = CalcDistance(endPos, pos);
                    break;
            }

            // 割合算出
            return (beginDist / (beginDist + endDist));
        }

        /// <summary>
        /// 2点間の距離を算出する。
        /// </summary>
        /// <param name="pos1">点1の位置。</param>
        /// <param name="pos2">点2の位置。</param>
        /// <returns>2点間の距離。</returns>
        double CalcDistance(Point pos1, Point pos2)
        {
            double dx = pos1.X - pos2.X;
            double dy = pos1.Y - pos2.Y;
            return Math.Sqrt(dx * dx + dy * dy);
        }

        /// <summary>
        /// 回転の基準点位置を算出する。ただし絶対指定の場合を除く。
        /// </summary>
        /// <param name="topLeftPos">対象の左上端の位置。</param>
        /// <param name="bmpSize">対象のサイズ。</param>
        /// <param name="type">回転の基準点位置種別。</param>
        /// <returns>基準点位置。絶対指定の場合は PointF.Empty 。</returns>
        PointF CalcRotateBasePoint(
            PointF topLeftPos, SizeF bmpSize, RotateBaseType type)
        {
            PointF rotPos = PointF.Empty;

            // X位置
            switch (type)
            {
                case RotateBaseType.TopLeft:
                case RotateBaseType.CenterLeft:
                case RotateBaseType.BottomLeft:
                    rotPos.X = topLeftPos.X;
                    break;

                case RotateBaseType.TopCenter:
                case RotateBaseType.Center:
                case RotateBaseType.BottomCenter:
                    rotPos.X = topLeftPos.X + bmpSize.Width * 0.5F;
                    break;

                case RotateBaseType.TopRight:
                case RotateBaseType.CenterRight:
                case RotateBaseType.BottomRight:
                    rotPos.X = topLeftPos.X + bmpSize.Width;
                    break;
            }

            // Y位置
            switch (type)
            {
                case RotateBaseType.TopLeft:
                case RotateBaseType.TopCenter:
                case RotateBaseType.TopRight:
                    rotPos.Y = topLeftPos.Y;
                    break;

                case RotateBaseType.CenterLeft:
                case RotateBaseType.Center:
                case RotateBaseType.CenterRight:
                    rotPos.Y = topLeftPos.Y + bmpSize.Height * 0.5F;
                    break;

                case RotateBaseType.BottomLeft:
                case RotateBaseType.BottomCenter:
                case RotateBaseType.BottomRight:
                    rotPos.Y = topLeftPos.Y + bmpSize.Height;
                    break;
            }

            return rotPos;
        }

        #region EffectBase メンバ

        public override string Category
        {
            get { return "チャンネル"; }
        }

        public override bool IsAudioEffect
        {
            get { return false; }
        }

        public override bool IsVideoEffect
        {
            get { return true; }
        }

        public override void Initialize(ILayer layer)
        {
            _layer = layer;
        }

        public override object SaveInnerData()
        {
            return null;
        }

        public override void LoadInnerData(object data)
        {
        }

        public override PropertyBase[] GetDefaultProperty()
        {
            SizeF size = _layer.GetImageBounds(0.0).Size;
            PointF basePos = new PointF(size.Width * 0.5F, size.Height * 0.5F);

            return new PropertyBase[]
                {
                    new LayerSelectableProperty("重ねるレイヤー"),
                    new SelectableProperty("ブレンド", 0, BlendTypeNames),
                    new NumberProperty("不透明度", 100.0, 100.0, 0.0),
                    new SelectableProperty("反転", (int)FlipType.None, FlipTypeNames),
                    // 配置
                        new VertexProperty(
                            "配:基準点", basePos, double.MaxValue, double.MinValue),
                        new SelectableProperty("配:横基準", 1, new string[]{ "左", "中央", "右" }),
                        new SelectableProperty("配:縦基準", 1, new string[]{ "上", "中央", "下" }),
                        new VertexProperty(
                            "配:スケール", new Size(100, 100), double.MaxValue, double.MinValue),
                    // 回転
                        new SelectableProperty(
                            "回:基準点", (int)RotateBaseType.Center, RotateBaseTypeNames),
                        new VertexProperty(
                            "回:基準点指定", basePos, double.MaxValue, double.MinValue),
                        new RadianProperty("回:回転量", 0, 0.0),
                    // 塗り潰し
                        new BooleanProperty("塗:有効", false),
                        new ColorProperty("塗:色", Color.Black),
                        new NumberProperty("塗:不透明度", 100.0, 100.0, 0.0),
                        // グラデーション
                            new BooleanProperty("塗:グ:有効", false),
                            new ColorProperty("塗:グ:色", Color.Black),
                            new NumberProperty("塗:グ:不透明度", 100.0, 100.0, 0.0),
                            new SelectableProperty(
                                "塗:グ:方向", (int)GradientDirection.TopToBottom, GradientDirNames),
                    // 時間指定
                        new SelectableProperty(
                            "時:指定方法", (int)RenderTimeType.None, RenderTimeTypeNames),
                        new NumberProperty("時:秒数", 0.0, double.MaxValue, double.MinValue),
                    // 品質
                        new BooleanProperty("品:補間処理", true),
                        new BooleanProperty("品:縮小フィルタ", false),
                };
        }

        public override PropertyEditControlBase[] GetControl()
        {
            return new PropertyEditControlBase[]
                {
                    new LayerSelectablePropertyEditControl(
                        "重ねるレイヤー", _layer.Composition, true, false, true, false),
                    new SelectablePropertyEditControl("ブレンド"),
                    new NumberPropertyEditControl(
                        "不透明度", 1.0, NumberPropertyEditControlType.Double),
                    new SelectablePropertyEditControl("反転"),
                    new PropertyNestBeginControl("配置"),
                        new VertexPropertyEditControl(
                            "配:基準点", 1.0, VertexPropertyEditControlType.PointFOrSizeF),
                        new SelectablePropertyEditControl("配:横基準"),
                        new SelectablePropertyEditControl("配:縦基準"),
                        new VertexPropertyEditControl(
                            "配:スケール", 1.0, VertexPropertyEditControlType.PointFOrSizeF),
                    new PropertyNestEndControl(),
                    new PropertyNestBeginControl("回転"),
                        new SelectablePropertyEditControl("回:基準点"),
                        new VertexPropertyEditControl(
                            "回:基準点指定", 1.0, VertexPropertyEditControlType.PointFOrSizeF),
                        new RadianPropertyEditControl("回:回転量"),
                    new PropertyNestEndControl(),
                    new PropertyNestBeginControl("塗り潰し"),
                        new BooleanPropertyEditControl("塗:有効"),
                        new ColorPropertyEditControl("塗:色"),
                        new NumberPropertyEditControl(
                            "塗:不透明度", 1.0, NumberPropertyEditControlType.Double),
                        new PropertyNestBeginControl("グラデーション"),
                            new BooleanPropertyEditControl("塗:グ:有効"),
                            new ColorPropertyEditControl("塗:グ:色"),
                            new NumberPropertyEditControl(
                                "塗:グ:不透明度", 1.0, NumberPropertyEditControlType.Double),
                            new SelectablePropertyEditControl("塗:グ:方向"),
                        new PropertyNestEndControl(),
                    new PropertyNestEndControl(),
                    new PropertyNestBeginControl("時間指定"),
                        new SelectablePropertyEditControl("時:指定方法"),
                        new NumberPropertyEditControl(
                            "時:秒数", 1.0, NumberPropertyEditControlType.Double),
                    new PropertyNestEndControl(),
                    new PropertyNestBeginControl("品質"),
                        new BooleanPropertyEditControl("品:補間処理"),
                        new BooleanPropertyEditControl("品:縮小フィルタ"),
                    new PropertyNestEndControl(),
                };
        }

        public override Roi CheckRoi(
            Roi roi,
            ReadOnlyDictionary<string, PropertyBase> property,
            double time)
        {
            // 特に変更しない
            return roi;
        }

        public override NBitmap ProcessingImage(
            NBitmap image,
            ReadOnlyDictionary<string, PropertyBase> property,
            Roi roi,
            double time)
        {
            // プロパティ取得
            int itemCode = ((LayerSelectableProperty)property["重ねるレイヤー"]).ItemCode;
            int blend = ((SelectableProperty)property["ブレンド"]).Selected;
            double alpha = ((NumberProperty)property["不透明度"]).DoubleValue;
            FlipType flipType = (FlipType)((SelectableProperty)property["反転"]).Selected;
            PointF basePos = ((VertexProperty)property["配:基準点"]).PointFValue;
            int horzIndex = ((SelectableProperty)property["配:横基準"]).Selected;
            int vertIndex = ((SelectableProperty)property["配:縦基準"]).Selected;
            SizeF scale = ((VertexProperty)property["配:スケール"]).SizeFValue;
            RotateBaseType rotateType =
                (RotateBaseType)((SelectableProperty)property["回:基準点"]).Selected;
            PointF rotatePos = ((VertexProperty)property["回:基準点指定"]).PointFValue;
            double rotateAngle = ((RadianProperty)property["回:回転量"]).Angle;
            bool fillEnable = ((BooleanProperty)property["塗:有効"]).Boolean;
            Color fillColor = ((ColorProperty)property["塗:色"]).Color;
            double fillAlpha = ((NumberProperty)property["塗:不透明度"]).DoubleValue;
            bool fillGradEnable = ((BooleanProperty)property["塗:グ:有効"]).Boolean;
            Color fillGradColor = ((ColorProperty)property["塗:グ:色"]).Color;
            double fillGradAlpha = ((NumberProperty)property["塗:グ:不透明度"]).DoubleValue;
            GradientDirection fillGradDir =
                (GradientDirection)((SelectableProperty)property["塗:グ:方向"]).Selected;
            RenderTimeType timeType =
                (RenderTimeType)((SelectableProperty)property["時:指定方法"]).Selected;
            double timeVal = ((NumberProperty)property["時:秒数"]).DoubleValue;
            bool interEnable = ((BooleanProperty)property["品:補間処理"]).Boolean;
            bool filterEnable = ((BooleanProperty)property["品:縮小フィルタ"]).Boolean;

            // 描画不要ならばそのまま返す
            if (alpha == 0.0 || scale.Width == 0.0 || scale.Height == 0.0)
            {
                return image;
            }

            // ソースレイヤー取得
            ILayer srcLayer = FindLayer(itemCode);
            if (srcLayer == null) { return image; }

            // ソース画像サイズに依存しない座標にレイヤー解像度を適用
            float resoRate = (float)_layer.ResolutionRate;
            basePos.X *= resoRate;
            basePos.Y *= resoRate;
            rotatePos.X *= resoRate;
            rotatePos.Y *= resoRate;

            // 取得時間設定
            double t = timeVal;
            if (timeType != RenderTimeType.Fixed)
            {
                t = srcLayer.ToLocalTime(_layer.ToWorldTime(time));
                if (timeType == RenderTimeType.Offset)
                {
                    t += timeVal;
                }
            }

            // イメージバッファサイズを算出
            SizeF bmpSize = srcLayer.GetImageBounds(t).Size;
            long bmpBufLen = (long)((bmpSize.Width + 1.0) * 4 * (bmpSize.Height + 1.0));

            // イメージを取得
            CacheManager.Manager.DemandTemporalMemory(bmpBufLen);
            using (NBitmap srcBmp = srcLayer.GetSourceImage(t))
            {
                CacheManager.Manager.AddWeakCache(srcBmp, srcBmp.DataSize);

                // 塗り潰し処理
                if (fillEnable)
                {
                    if (fillGradEnable)
                    {
                        FillBitmapByGradientColor(
                            srcBmp,
                            fillColor,
                            fillAlpha / 100.0,
                            fillGradColor,
                            fillGradAlpha / 100.0,
                            fillGradDir);
                    }
                    else
                    {
                        FillBitmapBySolidColor(srcBmp, fillColor, fillAlpha / 100.0);
                    }
                }

                // 不透明度補正
                if (alpha < 100.0)
                {
                    byte[] data = srcBmp.GetData();
                    Parallel.ForStep4(3, data.Length, (i) =>
                    {
                        data[i] = (byte)(data[i] * alpha / 100.0 + 0.5);
                    });
                }

                // 行列設定
                Matrix3D mat = new Matrix3D();
                {
                    float halfW = srcBmp.Width * 0.5F;
                    float halfH = srcBmp.Height * 0.5F;

                    // 中心を基準点にして反転設定
                    mat.Translate(-halfW, -halfH);
                    switch (flipType)
                    {
                        case FlipType.Horizontal:
                            mat.Scale(-1.0F, 1.0F);
                            break;

                        case FlipType.Vertical:
                            mat.Scale(1.0F, -1.0F);
                            break;

                        case FlipType.Both:
                            mat.Scale(-1.0F, -1.0F);
                            break;
                    }

                    // 配置の縦横基準位置を適用
                    float tx = 0.0F, ty = 0.0F;
                    switch (horzIndex)
                    {
                        case 0: tx = halfW; break;
                        case 1: break;
                        case 2: tx = -halfW; break;
                    }
                    switch (vertIndex)
                    {
                        case 0: ty = halfH; break;
                        case 1: break;
                        case 2: ty = -halfH; break;
                    }
                    mat.Translate(tx, ty);

                    // スケール設定
                    float scaleW = scale.Width / 100.0F, scaleH = scale.Height / 100.0F;
                    mat.Scale(scaleW, scaleH);

                    // 配置の基準点位置へ移動
                    mat.Translate(basePos.X, basePos.Y);

                    // 回転
                    if (rotateAngle != 0.0)
                    {
                        // 回転の基準点位置を決定
                        PointF rp = rotatePos;
                        if (rotateType != RotateBaseType.Absolute)
                        {
                            float dx = (tx - halfW) * scaleW;
                            float dy = (ty - halfH) * scaleH;
                            PointF topLeftPos = new PointF(basePos.X + dx, basePos.Y + dy);
                            SizeF sz = srcBmp.Size;
                            sz.Width *= scaleW;
                            sz.Height *= scaleH;
                            rp = CalcRotateBasePoint(topLeftPos, sz, rotateType);
                        }

                        // 回転設定
                        mat.RotateAt(rp.X, rp.Y, (float)rotateAngle);
                    }
                }

                // 描画
                BlendableGraphics g = new BlendableGraphics(image);
                {
                    // パラメータ設定
                    g.BlendMethod = blend;
                    g.Clip = roi.Interest;
                    g.Interpolation = interEnable;
                    g.Transform = mat;

                    // 描画
                    if (filterEnable)
                    {
                        g.FilteredDrawImage(srcBmp, PointF.Empty);
                    }
                    else
                    {
                        g.DrawImage(srcBmp, PointF.Empty);
                    }
                }
            }

            return image;
        }

        public override byte[] ProcessingAudio(
            byte[] audio,
            ReadOnlyDictionary<string, PropertyBase>[] property,
            double time)
        {
            // 呼ばれないはず
            throw new NotImplementedException();
        }

        #endregion

        #region PluginBase メンバ

        public override string PluginName
        {
            get { return "ブレンド(詳細)"; }
        }

        public override string Author
        {
            get { return "ルーチェ"; }
        }

        public override string InfoLink
        {
            get { return "http://www.ruche-home.net/"; }
        }

        public override string Description
        {
            get
            {
                return "パラメータを細かく指定してブレンド処理を行います。";
            }
        }

        #endregion

        #region IDisposable メンバ

        public override void Dispose()
        {
            // 何もしない
        }

        #endregion
    }
}
